@@ -2,12 +2,12 @@
 use strict;
 $| = 1;
 
-# $Id: archiverpt.pl 342 2003-08-07 19:15:33Z abeltje $
+# $Id: archiverpt.pl 858 2005-05-01 22:19:51Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.002';
+$VERSION = '0.004';
 
 use Cwd;
-use File::Spec;
+use File::Spec::Functions;
 use File::Path;
 use File::Copy;
 use FindBin;
@@ -118,21 +118,28 @@ $opt{adir} && -d $opt{adir} or do {
 my $patch_level = get_patch( $opt{ddir} );
 $patch_level =~ tr/ //sd;
 
-my $archived_rpt = "rpt${patch_level}.rpt";
+my $archived_rpt = catfile( $opt{adir}, "rpt${patch_level}.rpt" );
 
 # Do not archive if it is already done
--f File::Spec->catfile( $opt{adir}, $archived_rpt ) && $opt{force} and exit;
+-f $archived_rpt && !$opt{force} and exit;
 
-copy( File::Spec->catfile( $opt{ddir}, 'mktest.rpt' ), 
-      File::Spec->catfile( $opt{adir}, $archived_rpt ) ) or
-    die "Cannot copy to '$archived_rpt': $!";
+my $mktest_rpt = catfile( $opt{ddir}, 'mktest.rpt' );
+if ( -f $mktest_rpt ) {
+    copy( $mktest_rpt, $archived_rpt ) or
+        die "Cannot copy to '$archived_rpt': $!";
+}
 
 SKIP_LOG: {
     my $archived_log = "log${patch_level}.log";
-    last SKIP_LOG unless defined $opt{lfile};
-    last SKIP_LOG unless -f File::Spec->catfile( $FindBin::Bin, $opt{lfile} );
-    copy( File::Spec->catfile( $FindBin::Bin, $opt{lfile} ),
-          File::Spec->catfile( $opt{adir}, $archived_log ) ) or
+    unless ( defined $opt{lfile} ) {
+        $opt{v} and print "No logfile defined!\n";
+        last SKIP_LOG;
+    }
+    unless ( -f $opt{lfile} ) {
+        $opt{v} and print "Logfile '$opt{lfile}' not found!\n";
+        last SKIP_LOG;
+    }
+    copy( $opt{lfile}, File::Spec->catfile( $opt{adir}, $archived_log ) ) or
         die "Cannot copy to '$archived_log': $!";
 }
 
